﻿// -----------------------------------------------------------------------
// <copyright file="BrokenRule.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using System.Reflection;

namespace Spaanjaars.Validation
{
    /// <summary>
    /// The BrokenRule class provides (localized) information about the broken rules of validators.
    /// </summary>
    [Serializable]
    public class BrokenRule : IEquatable<BrokenRule>
    {

        #region Private Variables

        private string _propertyName = string.Empty;

        #endregion

        #region Constructor(s)

        /// <summary>
        /// Initializes a new instance of the BrokenRule class.
        /// </summary>
        /// <param name="propertyName">The name of the property that caused the rule to be broken.</param>
        /// <param name="message">The error message associated with the broken rule.</param>
        public BrokenRule(string propertyName, string message)
        {
            _propertyName = propertyName;
            Message = message;
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets the error message associated with the broken rule.
        /// </summary>
        /// <value>The localized error message.</value>
        public string Message { get; set; }

        /// <summary>
        /// Gets or sets the name of the property that caused the rule to be broken.
        /// </summary>
        /// <value>The name of the property.</value>
        public string PropertyName
        {
            get
            {
                return _propertyName;
            }
            set
            {
                if (value == null)
                {
                    value = string.Empty;
                }
                _propertyName = value;
            }
        }
        #endregion

        #region Public Methods

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            return string.Format("{0}: {1}", _propertyName, Message);
        }
        #endregion

        #region IEquatable
        /// <summary>
        /// Determines if two objects have the same values in all their
        /// public properties and fields.
        /// </summary>
        public bool Equals(BrokenRule other)
        {
            if (other == null) return base.Equals(other);

            // Make sure both classes are of the same type
            Type _t = this.GetType();
            if (!other.GetType().Equals(_t)) return false;

            // Compare all properties and ensure they're the same
            PropertyInfo[] props = _t.GetProperties();
            for (int i = 0; i < props.Length; i++)
            {
                object value1 = props[i].GetValue(this, new object[] { });
                object value2 = props[i].GetValue(other, new object[] { });
                
                // If both are null, or both are the same instance, skip to the next
                // iteration.
                if (ReferenceEquals(value1, null) && ReferenceEquals(value2, null))
                {
                    continue;
                }
                // If one is null, but not both, return false.
                if (ReferenceEquals(value1, null) || ReferenceEquals(value2, null))
                {
                    return false;
                }

                // If none are null, compare values.
                if (!value1.Equals(value2))
                {
                    return false;
                }
            }

            // Got this far - they must be identical!
            return true;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null) return base.Equals(obj);

            if (!(obj is BrokenRule))
                throw new InvalidCastException("The 'obj' argument is not a BrokenRule object.");
            else
                return Equals(obj as BrokenRule);
        }

        public override int GetHashCode()
        {
            int FinalHashCode = 17;
            int OtherCoPrimeNumber = 37;

            Type _t = this.GetType();

            // Compare all properties and ensure they're the same
            PropertyInfo[] props = _t.GetProperties();
            for (int i = 0; i < props.Length; i++)
            {
                object value1 = props[i].GetValue(this, new object[] { });

                // If value is null, skip to the next iteration.
                if (value1 != null)
                {
                    FinalHashCode = FinalHashCode * OtherCoPrimeNumber + value1.GetHashCode();
                }
            }

            return FinalHashCode;
        }

        public static bool operator ==(BrokenRule a, BrokenRule b)
        {
            // If both are null, or both are the same instance, return true.
            if (ReferenceEquals(a, null) && ReferenceEquals(b, null))
            {
                return true;
            }
            // If one is null, but not both, return false.
            if (ReferenceEquals(a, null) || ReferenceEquals(b, null))
            {
                return false;
            }
            // If neither is null, call the Equals method to compare properties.
            return a.Equals(b);
        }
        public static bool operator !=(BrokenRule a, BrokenRule b)
        {
            return !(a == b);
        }

        #endregion
    }
}

