﻿// -----------------------------------------------------------------------
// <copyright file="BrokenRulesCollection.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;

namespace Spaanjaars.Validation
{
    /// <summary>
    /// The BrokenRulesCollection is designed to hold <see cref="BrokenRule"/> items and supplies
    /// additional querying capabilities to retrieve specific BrokenRule instances.
    /// </summary>
    [Serializable]
    public class BrokenRulesCollection : Collection<BrokenRule>, IEquatable<BrokenRulesCollection>
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="BrokenRulesCollection"/> class.
        /// </summary>
        /// <param name="myList">My list.</param>
        internal BrokenRulesCollection(IList<BrokenRule> myList)
            : base(myList)
        { }

        /// <summary>
        /// Initializes a new instance of the <see cref="BrokenRulesCollection"/> class.
        /// </summary>
        public BrokenRulesCollection()
        { }


        #endregion

        #region Public Methods

        /// <summary>
        /// Returns a BrokenRulesCollection with the rules for the specified property name.
        /// </summary>
        /// <param name="propertyName">The (case insensitive) name of the property.</param>
        /// <returns>A BrokenRulesCollection for the specified property name. Returns an empty collection when the specified property name is not found.</returns>
        public BrokenRulesCollection FindByPropertyName(string propertyName)
        {
            return new BrokenRulesCollection((from rule in this
                                              where rule.PropertyName.ToUpperInvariant() == propertyName.ToUpperInvariant()
                                              select rule).ToList<BrokenRule>());
        }

        /// <summary>
        /// Returns a BrokenRulesCollection with the rules that contain (a part) of the specified message..
        /// </summary>
        /// <param name="message">The (case insensitive) part of the message to search for.</param>
        /// <returns>A BrokenRulesCollection containing the rules that match the specified message. Returns an empty collection when the specified message is not found.</returns>
        public BrokenRulesCollection FindByMessage(string message)
        {
            return new BrokenRulesCollection((from rule in this
                                              where rule.Message.ToUpperInvariant().Contains(message.ToUpperInvariant())
                                              select rule).ToList<BrokenRule>());
        }

        /// <summary>
        /// Creates a new BrokenRule instance and adds it to the inner list.
        /// </summary>
        /// <param name="message">The validation message associated with the broken rule.</param>
        public void Add(string message)
        {
            Add(new BrokenRule(string.Empty, message));
        }

        /// <summary>
        /// Creates a new BrokenRule instance and adds it to the inner list.
        /// </summary>
        /// <param name="propertyName">The name of the property that caused the rule to be broken. Can be left empty.</param>
        /// <param name="message">The validation message associated with the broken rule.</param>
        public void Add(string propertyName, string message)
        {
            Add(new BrokenRule(propertyName, message));
        }

        /// <summary>
        /// Returns a <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </summary>
        /// <returns>
        /// A <see cref="T:System.String"/> that represents the current <see cref="T:System.Object"/>.
        /// </returns>
        public override string ToString()
        {
            StringBuilder myStringBuilder = new StringBuilder();
            foreach (BrokenRule item in this)
            {
                myStringBuilder.Append(string.Format("{0}\r\n", item.ToString()));
            }
            return myStringBuilder.ToString();
        }

        /// <summary>
        /// Sorts the collection based on the specified comparer.
        /// </summary>
        /// <param name="comparer">The comparer that is used to sort this collection.</param>
        public void Sort(IComparer<BrokenRule> comparer)
        {
            if (comparer == null)
            {
                throw new ArgumentNullException("comparer", "Comparer is null.");
            }
            List<BrokenRule> list = this.Items as List<BrokenRule>;
            if (list == null)
            {
                return;
            }
            list.Sort(comparer);
        }

        #endregion

        #region IEquatable

        /// <summary>
        /// Determines if two collections have the same objects in their collections.
        /// </summary>
        public bool Equals(BrokenRulesCollection other)
        {
            if (other == null) return base.Equals(other);

            // Make sure both classes are of the same type
            Type _t = this.GetType();
            if (!other.GetType().Equals(_t)) return false;

            //Make sure both collections have the same number of items
            if (this.Count != other.Count) return false;

            // Declare a dictionary to count the occurence of the items in the collection        
            Dictionary<BrokenRule, int> itemCounts = new Dictionary<BrokenRule, int>();

            // Increase the count for each occurence of the item in the first collection        
            foreach (BrokenRule item in this)
            {
                if (itemCounts.ContainsKey(item))
                {
                    itemCounts[item]++;
                }
                else
                {
                    itemCounts[item] = 1;
                }
            }

            // Wrap the keys in a searchable list        
            List<BrokenRule> keys = new List<BrokenRule>(itemCounts.Keys);

            // Decrease the count for each occurence of the item in the second collection
            foreach (BrokenRule item in other)
            {
                // Try to find a key for the item
                // The keys of a dictionary are compared by reference, so we have to
                // find the original key that is equivalent to the "item"
                // You may want to override ".Equals" to define what it means for
                // two "T" objects to be equal
                BrokenRule key = keys.Find(
                    delegate(BrokenRule listKey)
                    {
                        return listKey.Equals(item);
                    });
                // Check if a key was found
                if (key != null)
                {
                    itemCounts[key]--;
                }
                else
                {
                    // There was no occurence of this item in the first collection, 
                    // thus the collections are not equal
                    return false;
                }
            }

            // The count of each item should be 0 if the contents of the collections are equal
            foreach (int value in itemCounts.Values)
            {
                if (value != 0)
                {
                    return false;
                }
            }

            // The collections are equal
            return true;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null) return base.Equals(obj);

            if (!(obj is BrokenRulesCollection))
                throw new InvalidCastException("The 'obj' argument is not a BrokenRulesCollection object.");
            else
                return Equals(obj as BrokenRulesCollection);
        }

        public override int GetHashCode()
        {
            int FinalHashCode = 17;
            int OtherCoPrimeNumber = 37;

            foreach (BrokenRule item in this)
            {
                FinalHashCode = FinalHashCode * OtherCoPrimeNumber + item.GetHashCode();
            }
            return FinalHashCode;
        }

        public static bool operator ==(BrokenRulesCollection a, BrokenRulesCollection b)
        {
            // If both are null, or both are the same instance, return true.
            if (ReferenceEquals(a, null) && ReferenceEquals(b, null))
            {
                return true;
            }
            // If one is null, but not both, return false.
            if (ReferenceEquals(a, null) || ReferenceEquals(b, null))
            {
                return false;
            }
            // If neither is null, call the Equals method to compare properties.
            return a.Equals(b);
        }
        public static bool operator !=(BrokenRulesCollection a, BrokenRulesCollection b)
        {
            return !(a == b);
        }

        #endregion

        
    }
}
