﻿// -----------------------------------------------------------------------
// <copyright file="BusinessBase.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using System.Resources;
using Spaanjaars.ContactManager.BusinessEntities.Localization;
using Spaanjaars.Validation;
using System.Reflection;

namespace Spaanjaars.ContactManager.BusinessEntities
{
    /// <summary>
    /// The BusinessBase class serves as the base class for all business entities in the SOM.DCO.MOGWAI.BusinessEntities namespace.
    /// Since it inheirts from ValidationBase, it provides validation behavior to its child classes. Additionally, it implements
    /// default behavior for concurrency checks.
    /// </summary>
    public abstract class BusinessBase : ValidationBase, IEquatable<BusinessBase>
    {
        /// <summary>
        /// The ID of the BusinessBase instance in the database.
        /// </summary>
        public abstract int Id { get; set; }

        /// <summary>
        /// Gets or sets a concurrency id that is used to keep track of changes made to the underlying data recor in the database.
        /// </summary>
        /// <value>The concurrency id.</value>
        public byte[] ConcurrencyId { get; set; }

        /// <summary>
        /// The ResuurceManager used by the validation framework.
        /// </summary>
        public static ResourceManager ResourceManager { get; set; }

        /// <summary>
        /// Gets the localized validation message based on the message key.
        /// </summary>
        /// <param name="key">The translation key of the validation message.</param>
        protected override string GetValidationMessage(string key)
        {
            string tempValue;
            if (ResourceManager != null)
            {
                tempValue = ResourceManager.GetString(key);
            }
            else
            {
                tempValue = string.Empty;
            }
            if (string.IsNullOrEmpty(tempValue))
            {
                tempValue = General.ResourceManager.GetString(key);
            }
            return tempValue;
        }

        #region IEquatable
        /// <summary>
        /// Determines if two objects have the same values in all their
        /// public properties and fields.
        /// </summary>
        public bool Equals(BusinessBase other)
        {
            if (other == null) return base.Equals(other);

            // Make sure both classes are of the same type
            Type _t = this.GetType();
            if (!other.GetType().Equals(_t)) return false;

            // Compare all properties and ensure they're the same
            PropertyInfo[] props = _t.GetProperties();
            for (int i = 0; i < props.Length; i++)
            {
                object value1 = props[i].GetValue(this, new object[] { });
                object value2 = props[i].GetValue(other, new object[] { });

                // If both are null, or both are the same instance, skip to the next
                // iteration.
                if (ReferenceEquals(value1, null) && ReferenceEquals(value2, null))
                {
                    continue;
                }
                // If one is null, but not both, return false.
                if (ReferenceEquals(value1, null) || ReferenceEquals(value2, null))
                {
                    return false;
                }

                // If none are null, compare values.
                if (!value1.Equals(value2))
                {
                    return false;
                }
            }

            // Got this far - they must be identical!
            return true;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null) return base.Equals(obj);

            if (!(obj is BusinessBase))
                throw new InvalidCastException("The 'obj' argument is not a BusinessBase object.");
            else
                return Equals(obj as BusinessBase);
        }

        public override int GetHashCode()
        {
            int FinalHashCode = 17;
            int OtherCoPrimeNumber = 37;

            Type _t = this.GetType();
            
            // Compare all properties and ensure they're the same
            PropertyInfo[] props = _t.GetProperties();
            for (int i = 0; i < props.Length; i++)
            {
                object value1 = props[i].GetValue(this, new object[] { });

                // If value is null, skip to the next iteration.
                if (value1 != null)
                {
                    FinalHashCode = FinalHashCode * OtherCoPrimeNumber + value1.GetHashCode();
                }
            }

            return FinalHashCode;
        }

        public static bool operator ==(BusinessBase a, BusinessBase b)
        {
            // If both are null, or both are the same instance, return true.
            if (ReferenceEquals(a, null) && ReferenceEquals(b, null))
            {
                return true;
            }
            // If one is null, but not both, return false.
            if (ReferenceEquals(a, null) || ReferenceEquals(b, null))
            {
                return false;
            }
            // If neither is null, call the Equals method to compare properties.
            return a.Equals(b);
        }
        public static bool operator !=(BusinessBase a, BusinessBase b)
        {
            return !(a == b);
        }

        #endregion
    }
}