﻿// -----------------------------------------------------------------------
// <copyright file="BusinessCollectionBase.cs" company="Imar.Spaanjaars.Com">
//   Copyright 2008 - 2009 - Imar.Spaanjaars.Com. All rights reserved.
// </copyright>
// -----------------------------------------------------------------------

using System;
using System.Collections.Generic;
using Spaanjaars.Validation;

namespace Spaanjaars.ContactManager.BusinessEntities
{
    /// <summary>
    /// The BusinessCollectionBase class serves as the base class for all main business entity collections.
    /// It overcomes limitations of the generic Collection&lt;T&gt; class by implementing a Sort method.
    /// </summary>
    /// <typeparam name="T">A class that inherits ValidationBase.</typeparam>
    public class BusinessCollectionBase<T> : ValidationCollectionBase<T>, IEquatable<BusinessCollectionBase<T>> where T : ValidationBase
    {
        /// <summary>
        /// Initializes a new instance of the BusinessCollectionBase class.
        /// </summary>
        public BusinessCollectionBase() { }

        /// <summary>
        /// Initializes a new instance of the BusinessCollectionBase class and populates it with the initial list.
        /// </summary>
        public BusinessCollectionBase(IList<T> initialList) : base(initialList) { }

        /// <summary>
        /// Sorts the collection based on the specified comparer.
        /// </summary>
        /// <param name="comparer">The comparer that is used to sort this collection.</param>
        public void Sort(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                throw new ArgumentNullException("comparer", "Comparer is null.");
            }
            List<T> list = this.Items as List<T>;
            if (list == null)
            {
                return;
            }
            list.Sort(comparer);
        }

        #region IEquatable

        /// <summary>
        /// Determines if two collections have the same objects in their collections.
        /// </summary>
        public bool Equals(BusinessCollectionBase<T> other)
        {
            if (other == null) return base.Equals(other);

            // Make sure both classes are of the same type
            Type _t = this.GetType();
            if (!other.GetType().Equals(_t)) return false;

            //Make sure both collections have the same number of items
            if (this.Count != other.Count) return false;

            // Declare a dictionary to count the occurence of the items in the collection        
            Dictionary<T, int> itemCounts = new Dictionary<T, int>();

            // Increase the count for each occurence of the item in the first collection        
            foreach (T item in this)
            {
                if (itemCounts.ContainsKey(item))
                {
                    itemCounts[item]++;
                }
                else
                {
                    itemCounts[item] = 1;
                }
            }

            // Wrap the keys in a searchable list        
            List<T> keys = new List<T>(itemCounts.Keys);

            // Decrease the count for each occurence of the item in the second collection
            foreach (T item in other)
            {
                // Try to find a key for the item
                // The keys of a dictionary are compared by reference, so we have to
                // find the original key that is equivalent to the "item"
                // You may want to override ".Equals" to define what it means for
                // two "T" objects to be equal
                T key = keys.Find(
                    delegate(T listKey)
                    {
                        return listKey.Equals(item);
                    });
                // Check if a key was found
                if (key != null)
                {
                    itemCounts[key]--;
                }
                else
                {
                    // There was no occurence of this item in the first collection, 
                    // thus the collections are not equal
                    return false;
                }
            }

            // The count of each item should be 0 if the contents of the collections are equal
            foreach (int value in itemCounts.Values)
            {
                if (value != 0)
                {
                    return false;
                }
            }

            // The collections are equal
            return true;
        }

        public override bool Equals(Object obj)
        {
            if (obj == null) return base.Equals(obj);

            if (!(obj is BusinessCollectionBase<T>))
                throw new InvalidCastException("The 'obj' argument is not a BusinessCollectionBase<T> object.");
            else
                return Equals(obj as BusinessCollectionBase<T>);
        }

        public override int GetHashCode()
        {
            int FinalHashCode = 17;
            int OtherCoPrimeNumber = 37;

            foreach (T item in this)
            {
                FinalHashCode = FinalHashCode * OtherCoPrimeNumber + item.GetHashCode();
            }
            return FinalHashCode;
        }

        public static bool operator ==(BusinessCollectionBase<T> a, BusinessCollectionBase<T> b)
        {
            // If both are null, or both are the same instance, return true.
            if (ReferenceEquals(a, null) && ReferenceEquals(b, null))
            {
                return true;
            }
            // If one is null, but not both, return false.
            if (ReferenceEquals(a, null) || ReferenceEquals(b, null))
            {
                return false;
            }
            // If neither is null, call the Equals method to compare properties.
            return a.Equals(b);
        }
        public static bool operator !=(BusinessCollectionBase<T> a, BusinessCollectionBase<T> b)
        {
            return !(a == b);
        }

        #endregion
    }
}
